from time import sleep
from SX127x.LoRa import *
from SX127x.board_config import BOARD

print("\nThe very beginning...\n")
BOARD.setup()

class LoRaRcvCont(LoRa):
    def __init__(self, verbose=False):
        super(LoRaRcvCont, self).__init__(verbose)
        self.set_mode(MODE.SLEEP)
        self.set_dio_mapping([0] * 6)

    def start(self):
        self.reset_ptr_rx()
        self.set_mode(MODE.RXCONT)
        while True:
            sleep(.5)
            rssi_value = self.get_rssi_value()
            status = self.get_modem_status()
            sys.stdout.flush()
            
    def on_rx_done(self):
        print("\nReceived: ")
        self.clear_irq_flags(RxDone=1)
        payload = self.read_payload(nocheck=True)
        try:
            print(bytes(payload).hex())
        except (IndexError):
            print("Packet Index Error: Most likely a transmission collision")
        self.print_packet(payload)
        self.set_mode(MODE.SLEEP)
        self.reset_ptr_rx()
        self.set_mode(MODE.RXCONT) 

    def print_packet(self, payload):
        import struct
        
        print("RSSI: ", self.get_rssi_value(), "    Pkt RSSI: ", self.get_pkt_rssi_value(), "    Pkt SNR: ", self.get_pkt_snr_value())
        
        print("\nDestination MAC : ", end =" ")
        for i in range(0,4):
            print('{:02x}'.format(payload[i]), end =" ")
        destinationNode = int.from_bytes(payload[0:4], byteorder='little')
        if destinationNode == 0xa16ef174:
            print("\n                :  LoRa Gateway", end =" ")
        else:
            print("\n                :  Unrecognised Destination", end =" ")
            
        print("\n     Source MAC : ", end =" ")
        for i in range(4,8):
            print('{:02x}'.format(payload[i]), end =" ")
        sourceNode = int.from_bytes(payload[4:8], byteorder='little')
        if sourceNode == 0xa16ef174:
            print("\n                :  LoRa Gateway", end =" ")
        elif sourceNode == 0xa16ef75c:
            print("\n                :  Node 32 Power", end =" ")
        elif sourceNode == 0xa1125785:
            print("\n                :  Rover ESP32 BME", end =" ")
        elif sourceNode == 0xa1517110:
            print("\n                :  Stick Rainfall", end =" ")
        elif sourceNode == 0xa151739c:
            print("\n                :  Stick Tank Level", end =" ")
        elif sourceNode == 0xa1c00001:
            print("\n                :  Cube Wind", end =" ")
        elif sourceNode == 0xa1c00002:
            print("\n                :  Cube BME", end =" ")
        elif sourceNode == 0xa1e00001:
            print("\n                :  Breadboard Mini BME", end =" ")
        elif sourceNode == 0xa1e00002:
            print("\n                :  Zweirs Mini BME", end =" ")
        elif sourceNode == 0xa1e00003:
            print("\n                :  Porch Mini BME", end =" ")
        elif sourceNode == 0xa1e00004:
            print("\n                :  Shed Mini BME", end =" ")
        else:
            print("\n                :  Unrecognised Source", end =" ")
            
        print("\n            CRC : ", end =" ")
        for i in range(8,12):
            print('{:02x}'.format(payload[i]), end =" ")
            
        print("\n     Sequence # : ", end =" ")
        for i in range(12,14):
            print('{:02x}'.format(payload[i]), end =" ")
        print("\n                : ", int.from_bytes(payload[12:14], byteorder='little'), end =" ")
        
        print("\n           Type : ", '{:02x}'.format(payload[14]))        
        validType = True
        if payload[14] == 0x00:
            print("                :  Power", end =" ")
        elif payload[14] == 0x01:
            print("                :  Voltage", end =" ")
        elif payload[14] == 0x11:
            print("                :  Tank", end =" ")
        elif payload[14] == 0x20:
            print("                :  Weather", end =" ")
        elif payload[14] == 0x21:
            print("                :  Atmosphere", end =" ")
        elif payload[14] == 0x22:
            print("                :  Rainfall", end =" ")
        elif payload[14] == 0x23:
            print("                :  Wind", end =" ")
        else:
            print("                :  Unrecognised", end =" ")
            validType = False
            
        if validType:    
            print("\n Payload Length : ", '{:02x}'.format(payload[15]))
            print("                : ", int.from_bytes(payload[15:16], byteorder='little'))
            print("        Payload : ", end =" ")
            for i in range(16,(16+int.from_bytes(payload[15:16], byteorder='little'))):
                print('{:02x}'.format(payload[i]), end =" ")
            print("")
            if payload[14] == 0x00:
                print("Battery Voltage : ", "{:.2f}".format(int.from_bytes(payload[16:18], byteorder='little')/1000), "V")
                print("Battery Current : ", int.from_bytes(payload[18:20], byteorder='little'), "mA")
                print("  Panel Voltage : ", "{:.2f}".format(int.from_bytes(payload[20:22], byteorder='little')/1000), "V")
                print("  Panel Current : ", int.from_bytes(payload[22:24], byteorder='little'), "mA")
                print("   Load Voltage : ", "{:.2f}".format(int.from_bytes(payload[24:26], byteorder='little')/1000), "V")
                print("   Load Current : ", int.from_bytes(payload[26:28], byteorder='little'), "mA", end =" ")
            elif payload[14] == 0x01:
#                print("                : ","Voltage")
                print("        Voltage : ", "{:.2f}".format(int.from_bytes(payload[16:18], byteorder='little')/1000), "V", end =" ")
            elif payload[14] == 0x11:
#                print("                : ","Tank")
                print("          Level : ", int.from_bytes(payload[16:17], byteorder='little'), "%", end =" ")
            elif payload[14] == 0x20:
#                print("                : ","Weather")
                print("    Temperature : ", "{:.1f}".format(int.from_bytes(payload[16:18], byteorder='little')/10), "°C")
                print("       Pressure : ", int.from_bytes(payload[18:20], byteorder='little'), "hPa")
                print("       Humidity : ", int.from_bytes(payload[20:22], byteorder='little'), "%")
                print("       Rainfall : ", "{:.1f}".format(int.from_bytes(payload[22:24], byteorder='little')/10), "mm")
                print(" Wind Direction : ", int.from_bytes(payload[24:26], byteorder='little'), "°")
                print("     Wind Speed : ", int.from_bytes(payload[26:28], byteorder='little'), "kph", end =" ")
            elif payload[14] == 0x21:
#                print("                : ","Atmosphere")
                print("    Temperature : ", "{:.1f}".format(int.from_bytes(payload[16:18], byteorder='little')/10), "°C")
                print("       Pressure : ", int.from_bytes(payload[18:20], byteorder='little'), "hPa")
                print("       Humidity : ", int.from_bytes(payload[20:22], byteorder='little'), "%", end =" ")
            elif payload[14] == 0x22:
#                print("                : ","Rainfall")
                print("       Rainfall : ", "{:.1f}".format(int.from_bytes(payload[16:18], byteorder='little')/10), "mm", end =" ")
            elif payload[14] == 0x23:
#                print("                : ","Wind")
                print("      Direction : ", int.from_bytes(payload[16:18], byteorder='little'), "°")
                print("          Speed : ", int.from_bytes(payload[18:19], byteorder='little'), "kph", end =" ")
            print("\n")

lora = LoRaRcvCont(verbose=True)
lora.set_mode(MODE.STDBY)

#  Medium Range  Defaults after init should be 917.0MHz, Bw = 125 kHz, Cr = 4/5, Sf = 128chips/symbol, CRC on 13 dBm

lora.set_pa_config(pa_select=1)
lora.set_freq(917)
print("Set Frequency: ",lora.get_freq(), "MHz")
lora.set_sync_word(0x12)
print("Set Sync Word: ",hex(lora.get_sync_word()))

try:
    lora.start()
except KeyboardInterrupt:
    sys.stdout.flush()
    print("")
    sys.stderr.write("KeyboardInterrupt\n")
finally:
    sys.stdout.flush()
    print("")
    lora.set_mode(MODE.SLEEP)
    BOARD.teardown()
